Package: renv
Type: Package
Title: Project Environments
Version: 1.2.3
Authors@R: c(
    person("Kevin", "Ushey", role = c("aut", "cre"), email = "kevin@rstudio.com",
           comment = c(ORCID = "0000-0003-2880-7407")),
    person("Hadley", "Wickham", role = c("aut"), email = "hadley@rstudio.com",
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
    )
Description: A dependency management toolkit for R. Using 'renv', you can create
    and manage project-local R libraries, save the state of these libraries to
    a 'lockfile', and later restore your library as required. Together, these
    tools can help make your projects more isolated, portable, and reproducible.
License: MIT + file LICENSE
URL: https://rstudio.github.io/renv/, https://github.com/rstudio/renv
BugReports: https://github.com/rstudio/renv/issues
Imports: utils
Suggests: BiocManager, cli, compiler, covr, cpp11, curl, devtools,
        generics, gitcreds, jsonlite, jsonvalidate, knitr, miniUI,
        modules, packrat, pak, R6, remotes, reticulate, rmarkdown,
        rstudioapi, shiny, testthat, uuid, waldo, yaml, webfakes
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/testthat/start-first:
        bioconductor,python,install,restore,snapshot,retrieve,remotes
NeedsCompilation: no
Packaged: 2026-05-16 18:11:48 UTC; kevin
Author: Kevin Ushey [aut, cre] (ORCID: <https://orcid.org/0000-0003-2880-7407>),
  Hadley Wickham [aut] (ORCID: <https://orcid.org/0000-0003-4757-117X>),
  Posit Software, PBC [cph, fnd]
Maintainer: Kevin Ushey <kevin@rstudio.com>
Repository: CRAN
Date/Publication: 2026-05-16 21:50:15 UTC
Built: R 4.6.0; ; 2026-05-17 02:15:01 UTC; unix
