Package: renv
Version: 1.2.3
Imports: utils
Suggests: BiocManager, cli, compiler, covr, cpp11, curl, devtools,
        generics, gitcreds, jsonlite, jsonvalidate, knitr, miniUI,
        modules, packrat, pak, R6, remotes, reticulate, rmarkdown,
        rstudioapi, shiny, testthat, uuid, waldo, yaml, webfakes
License: MIT + file LICENSE
MD5sum: 9bf33bcb88ed6cf8309ce19358fea014
NeedsCompilation: no
