Package: windowsonly
Type: Package
Version: 1.0.0
Repository: CRAN
License: GPL
Description: renv test package
Title: renv test package
Author: Anonymous Person <Anonymous@rstudio.org>
Maintainer: Anonymous Person <Anonymous@rstudio.org>
OS_type: windows
