Package: today
Type: Package
Version: 1.0.0
Depends: R (>= 99.99.99)
Repository: CRAN
License: GPL
Description: renv test package
Title: renv test package
Author: Anonymous Person <Anonymous@rstudio.org>
Maintainer: Anonymous Person <Anonymous@rstudio.org>
