Package: renv
Version: 1.2.2
Imports: utils
Suggests: BiocManager, cli, compiler, covr, cpp11, curl, devtools,
        generics, gitcreds, jsonlite, jsonvalidate, knitr, miniUI,
        modules, packrat, pak, R6, remotes, reticulate, rmarkdown,
        rstudioapi, shiny, testthat, uuid, waldo, yaml, webfakes
License: MIT + file LICENSE
MD5sum: e2361e1cf108b82326c8a826b7f0798b
NeedsCompilation: no
