# Generated by roxygen2: do not edit by hand

export(activate)
export(autoload)
export(checkout)
export(clean)
export(config)
export(consent)
export(deactivate)
export(dependencies)
export(diagnostics)
export(embed)
export(equip)
export(history)
export(hydrate)
export(init)
export(install)
export(isolate)
export(load)
export(lockfile_create)
export(lockfile_modify)
export(lockfile_read)
export(lockfile_validate)
export(lockfile_write)
export(migrate)
export(modify)
export(paths)
export(plan)
export(project)
export(purge)
export(rebuild)
export(record)
export(refresh)
export(rehash)
export(remove)
export(repair)
export(restore)
export(retrieve)
export(revert)
export(run)
export(sandbox)
export(scaffold)
export(settings)
export(snapshot)
export(status)
export(sysreqs)
export(update)
export(upgrade)
export(use)
export(use_python)
export(vulns)
importFrom(tools,file_ext)
importFrom(tools,md5sum)
importFrom(tools,package_dependencies)
importFrom(tools,pskill)
importFrom(tools,psnice)
importFrom(tools,write_PACKAGES)
importFrom(utils,Rprof)
importFrom(utils,URLencode)
importFrom(utils,adist)
importFrom(utils,available.packages)
importFrom(utils,browseURL)
importFrom(utils,citation)
importFrom(utils,contrib.url)
importFrom(utils,download.file)
importFrom(utils,download.packages)
importFrom(utils,file.edit)
importFrom(utils,getCRANmirrors)
importFrom(utils,head)
importFrom(utils,help)
importFrom(utils,install.packages)
importFrom(utils,installed.packages)
importFrom(utils,modifyList)
importFrom(utils,old.packages)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,read.table)
importFrom(utils,remove.packages)
importFrom(utils,sessionInfo)
importFrom(utils,str)
importFrom(utils,summaryRprof)
importFrom(utils,tail)
importFrom(utils,tar)
importFrom(utils,toBibtex)
importFrom(utils,untar)
importFrom(utils,unzip)
importFrom(utils,update.packages)
importFrom(utils,zip)
