
## Parse Errors

renv should still be able to recover dependencies from the chunks without
parse errors.

```{r}
library(dplyr)
```

```{r}
this chunk has parse errors
```

```{r}
and so does this chunk
```

```{r}
we forgot to close this chunk
