% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manifest-convert.R
\name{renv_lockfile_from_manifest}
\alias{renv_lockfile_from_manifest}
\title{Generate \code{renv.lock} from an Posit Connect \code{manifest.json}}
\usage{
renv_lockfile_from_manifest(
  manifest = "manifest.json",
  lockfile = NA,
  project = NULL
)
}
\arguments{
\item{manifest}{The path to a \code{manifest.json} file.}

\item{lockfile}{The path to the lockfile to be generated and / or updated.
When \code{NA} (the default), the generated lockfile is returned as an \R
object; otherwise, the lockfile will be written to the path specified by
\code{lockfile}.}
}
\value{
An renv lockfile.
}
\description{
Use \code{renv_lockfile_from_manifest()} to convert a \code{manifest.json} file from
an Posit Connect content bundle into an \code{renv.lock} lockfile.

This function can be useful when you need to recreate the package environment
of a piece of content that is deployed to Posit Connect. The content bundle
contains a \code{manifest.json} file that is used to recreate the package
environment. This function will let you convert that manifest file to an
\code{renv.lock} file. Run \code{renv::restore()} after you've converted the file to
restore the package environment.
}
\keyword{internal}
